﻿# -*- coding: utf_8 -*-

from clconst import *
from clapi import *
from clbmitem_tag import ItemTag as Tag
from clbmitem_url import ItemUrl as Url
from clbmitem_cmd import ItemCmd as Cmd
from itertools import imap
try:
    import clKanji
except:
    pass

from time import time

# ------------------------------------------------------------------ 
# support function                                                  |
# ------------------------------------------------------------------ 
# 重複を除去したリストを返す
def uniq(x):
    return list(set(x))

#全要素を小文字化したリストを返す
def lower(x):
    def support(y):
        return y.lower()
    return list(imap(support,iter(x)))

# オブジェクトの __で始まらないメソッド,変数のリストを返す
def dirEx(obj):
    return [item for item in obj.__class__.__dict__.keys() if not item.startswith('__')]

class Item:
    u"""操作用クラス"""
    
    # 高速化の為のキャッシュ
    tags = []
    urls = []
    cmds = []
    members = set()
    key = ''
    tags_sel = []
    change_flag = 0
    sort_type = GetOption(OPTION_BM_SORT_BY_UPDATE)
    
    def abbrev(tags_sel, key=None):
        sort_type = GetOption(OPTION_BM_SORT_BY_UPDATE)
        urls = []
        tags = []
        cmds = []
        if Item.change_flag:
            Item.tags = []
            Item.urls = []
            Item.key = ''
            Item.tags_sel = []
            Item.change_flag = 0
        
        if key:
            if Item.key and Item.tags_sel and Item.key in key and Item.tags_sel <= tags_sel:
                
                # key で部分一致検索(url)
                urls = [url for url in Item.urls if key.lower() in '\n'.join(url.kanas).lower()]
                
                # key で部分一致検索(tag)
                tags = [tag for tag in Item.tags if key.lower() in '\n'.join(tag.kanas).lower()]
                
                # key で部分一致検索(cmd)
                cmds = [cmd for cmd in Item.cmds if key.lower() in '\n'.join(cmd.kanas).lower()]
            else:
                # key で部分一致検索(url)
                urls = [url for url in Url._items if key.lower() in '\n'.join(url.kanas).lower() and tags_sel <= set(url.tags)]
                
                # key で部分一致検索(tag)
                tags = [tag for tag in Tag._items if key.lower() in '\n'.join(tag.kanas).lower()]
                
                # key で部分一致検索(tag)
                cmds = [cmd for cmd in Cmd._items if key.lower() in '\n'.join(cmd.kanas).lower()]
                
            Item.key = key
            
            # sort
            if urls:
                Item.urls = urls = Item.getUrls(sort_type, urls)
                
            if tags:
                Item.tags = tags = Item.getTags(sort_type, tags)
            else:
                # urlに関連するタグ
                urltags = [tag for url in urls for tag in url.tags]
                urltags = [tag for tag in Tag._items for urltag in urltags if urltag in '\n'.join(tag.member)]
                tags = uniq([urltag for urltag in urltags if not urltag in tags])
            if cmds:
                Item.cmds = cmds = Item.getCmds(sort_type, cmds)
                
        else:
            if tags_sel:
                urls = [url for url in Url._items if tags_sel <= set(url.tags)]
                
                if urls:
                    Item.urls = urls = Item.getUrls(sort_type, urls)
                    
                Item.tags = tags = Item.getTags(sort_type, tags)
                Item.cmds = cmds = Item.getCmds(sort_type, cmds)
                Item.tags_sel = tags_sel
                
            else:
                # sort
                Item.urls = urls = Item.getUrls(sort_type, urls)
                Item.tags = tags = Item.getTags(sort_type, tags)
                Item.cmds = cmds = Item.getCmds(sort_type, cmds)
                
        
        return urls, tags, cmds
    abbrev = staticmethod(abbrev)
    
    def find(key, param , obj_type=''):
        items = []
        if param in ['url', 'description', 'tags', 'rss'] or obj_type == 'url':
            items = Url._items
        elif param == 'member' or obj_type == 'tag':
            items = Tag._items
        elif param == 'id' or obj_type == 'cmd':
            items = Cmd._items
        elif param in ['text', 'kanas'] and obj_type == '':
            items = Url._items + Tag._items + Cmd._items
        else:
            return [None]
        if not items:
            return [None]
        
        if param in ['tags','kanas','member']:
            res = [item for item in items if key in lower(getattr(item,param))]
        elif param in ['url','open','description','rss','text','id']:
            res = [item for item in items if getattr(item,param) == key]
            
        if not res:
            return [None]
        return res
    find = staticmethod(find)
    
    def removeAll():
        Url._items = []
        Tag._items = []
        Cmd._items = []
    removeAll = staticmethod(removeAll)
    
    def setCmdFuncMap(id, func):
        Cmd.func_map[id] = func
    setCmdFuncMap = staticmethod(setCmdFuncMap)
    
    # -------------------------------------------------------------- 
    # sort and get items                                            |
    # -------------------------------------------------------------- 
    
    def getAll():
        sort_type = GetOption(OPTION_BM_SORT_BY_UPDATE)
        Item.urls = urls = Item.getUrls(sort_type)
        Item.tags = tags = Item.getTags(sort_type)
        Item.cmds = cmds = Item.getCmds(sort_type)
        Item.key = ''
        
        return iter(tags + urls + cmds)
    getAll = staticmethod(getAll)
    
    # sort and return list.
    def getUrls(sort_type, items=None):
        
        if items:
            urls = items
        else:
            urls = Url._items
        
        # url.up > url.frequency > url.visit
        if sort_type:
            urls = [(url.up, url.frequency ,url.visit ,url) for url in urls]
        # url.frequency > url.up > url.visit
        else:
            urls = [(url.frequency, url.up, url.visit ,url) for url in urls]
            
        res = [url for (a,b,c, url) in reversed(sorted(urls))]
        
        if not items:
            Url._items = res
            
        return res
    getUrls = staticmethod(getUrls)
    
    # sort and return list(tag objects)
    def getTags(sort_type, items=None):
        
        if items:
            tags = items
        else:
            tags = Tag._items
        
        # tag.frequency > tag.visit
        if sort_type:
            tags = [(tag.frequency, tag.visit ,tag) for tag in tags]
        # tag.visit > tag.frequency
        else:
            tags = [(tag.visit ,tag.frequency, tag) for tag in tags]
            
        res = [tag for (a,b, tag) in reversed(sorted(tags))]
        
        if not items:
            Tag._items = res
            
        return res
    getTags = staticmethod(getTags)
    
    # sort and return list(cmd objects)
    def getCmds(sort_type, cmds=None):
        if not cmds:
            cmds = Cmd._items
        # tag.frequency > tag.visit
        if sort_type:
            cmds = [(cmd.frequency, cmd.visit ,cmd) for cmd in cmds]
        # tag.visit > tag.frequency
        else:
            cmds = [(cmd.visit ,cmd.frequency, cmd) for cmd in cmds]
        return [cmd for (a,b, cmd) in reversed(sorted(cmds))]
    getCmds = staticmethod(getCmds)
    
    def getAllMember():
        #if Item.change_flag:
        #    Item.members = set(lower([member for tag in Tag._items for member in tag._member]))
        #return Item.members
        return set(lower([member for tag in Tag._items for member in tag.member]))
    getAllMember = staticmethod(getAllMember)
    
    # -------------------------------------------------------------- 
    # set item from import xxx                                      |
    # -------------------------------------------------------------- 
    
    def setItemUrl(text, kanas, url, open, description, tags, up=0, rss=u''):
        #要素不足
        item = Item.find(url,'url','url')[0]
        frequency = 0
        visit = 0
        if item:
            frequency = item.frequency
            visit = item.visit
            
        if not text:
            print '%s has no text.' % url
            return
        else:
            text = unicode(text)
            
        if not url:
            print '%s has no url.' % text
            return
        else:
            url = unicode(url)
            
        if not kanas:
            if item:
                kanas = item.kanas
            else:
                kanas = [clKanji.toRoman(clKanji.ReConv(text)[0])]
                
        if not description:
            if item:
                description = unicode(item.description)
            else:
                description = u''
                
        if not open:
            if item:
                open = item.open
            else:
                open = url
                
        if not up:
            if item:
                up = item.up
            else:
                up = 0
                
        if tags:
            tags = [unicode(tag) for tag in tags]
            
        if rss:
            rss = unicode(rss)
        
        Url(text, kanas, url, open, description, tags, rss, frequency, visit, up).add()
        
    setItemUrl = staticmethod(setItemUrl)
    
    def setItemTag(text, member, kanas):
        item = Item.find(text,'text','tag')[0]
        frequency = 0
        visit = 0
        if item:
            frequency = item.frequency
            visit = item.visit
        
        if not text:
            print '%s (%s) has no text.' % (kanas, member)
            return
        else:
            text = unicode(text)
            
        if not member:
            if item:
                member = item.member
            else:
                member = [text]
                
        if not kanas:
            if item:
                kanas = item.kanas
            else:
                kanas = [clKanji.toRoman(clKanji.ReConv(text)[0])]
        Tag(text, kanas, member, visit, frequency).add()
    setItemTag = staticmethod(setItemTag)
    
    def setItemCmd(id, text, kanas):
        
        item = Item.find(id,'id','cmd')[0]
        if item:
            visit = item.visit
            frequency = item.frequency
        else:
            frequency = 0
            visit = 0
        
        if not id:
            print '%s has no id.' % text
        if not text:
            text = item.text
        if not kanas:
            kanas = item.kanas
        Cmd(id, text, kanas, frequency, visit).add()
    setItemCmd = staticmethod(setItemCmd)
    
