﻿# -*- coding: utf_8 -*-
from clbmconst import *
from clconst import *
from clapi import *

import os
import time
import clevent
import clmode
from types import *

# ------------------------------------------------------------------ 
# Item base class                                                   |
# ------------------------------------------------------------------ 

class BaseItem:
    def set(self, _text=u'', _kanas=[u''], _frequency=0, _visit=0):
        self.text = _text
        self.kanas = _kanas
        self.frequency = _frequency
        self.visit = _visit
        self.add()
        
    def __call__(self):
        self.frequency += 1
        self.visit = int(time.strftime("%Y%m%d%H%M",time.localtime()))
        # ex) 2005 01/02 03:04
        #     200501020304
        
    def baseInfo(self):
        res = []
        res.append('text : ' + self.text)
        res.append('kanas : ' + '|'.join(self.kanas))
        res.append('frequency : ' + str(self.frequency))
        res.append('visit : ' + str(self.visit))
        return '\n'.join(res)
    
    def add(self, nocheck = False):
        if nocheck:
            self.__class__._items.append(self)
            return 
        
        res = self.checkDuplication()
        if not res:
            self.__class__._items.append(self)
            return
        elif res == 1: 
            pass
        else:
            try:
                self.displayDuplicationMsg(res)
            except ItemImportCancel:
                raise ItemImportCancel
        
    def remove(self):
        self.__class__._items.remove(self)
        return
    
    def checkDuplication(self):
        pass
    
    def displayDuplicationMsg(self,old):
        msg,msgtitle = self.makeOverwriteMsg(old)
        msg = ''.join(msg)
        msgtitle = ''.join(msgtitle)
        result = MessageBox(msg, msgtitle, MB_YESNOCANCEL)
        if result == IDYES:
            old.remove()
            self.__class__._items.append(self)
        elif result == IDNO:
            return
        elif result == IDCANCEL:
            raise ItemImportCancel
    
    def makeOverwriteMsg(self, old):#new(new item object), old(old item object)
        msg = [u'「はい」を押すと*マークのついた部分が上段から下段へ変更されます。\n']
        msg.append(u'---------------------------------------------------------------\n')
        msgtitle = u''
        
        _otext = old.text
        _ntext = self.text
        msg.extend(['text : ' + _otext + '\n','text : ' + _ntext + '\n\n'])
        if not _ntext.lower() == _otext.lower():
            msg[-1], msg[-2] = '* '+msg[-1], '* '+msg[-2]
            
        _okanas = '|'.join(old.kanas)
        _nkanas = '|'.join(self.kanas)
        msg.extend(['kana : ' + _okanas + '\n','kana : ' + _nkanas + '\n\n'])
        if not _nkanas.lower() == _okanas.lower():
            msg[-1], msg[-2] = '* '+msg[-1], '* '+msg[-2]
            
        _ofrequency = str(old.frequency)
        _nfrequency = str(self.frequency)
        msg.extend(['open : ' + _ofrequency + '\n','open : ' + _nfrequency + '\n\n'])
        if not _nfrequency == _ofrequency:
            msg[-1], msg[-2] = '* '+msg[-1], '* '+msg[-2]
            
        _ovisit = str(old.visit)
        _nvisit = str(self.visit)
        msg.extend(['visit : ' + _ovisit + '\n','visit : ' + _nvisit + '\n\n'])
        if not _nvisit == _ovisit:
            msg[-1], msg[-2] = '* '+msg[-1], '* '+msg[-2]
            
        return msg,msgtitle
