﻿# -*- coding: utf_8 -*-

from clconst import *
from clapi import *
from clbmitem_base import BaseItem as Base
import time

# ------------------------------------------------------------------ 
# Item url class                                                    |
# ------------------------------------------------------------------ 
"""
text        str   サイトの名前 (view)
kanas       [str] ローマ字表記された名前 (search)
frequency   int   開いた回数 (sort)
visit       int   最後に開いた時間 (sort)

url         str   URL (duplicate check)
open        str   URL (execute)
description str   概要 (view)
tags        [str] タグ (sort)
up          int   更新時間 (sort)
rss         str   RSSのURL (なんとなく)

正しい引数を与えれば値を設定。
"""

class ItemUrl(Base):
    u"""this class deal with url item."""
    
    _items = []
    
    def __init__(self, text=u'', kanas=[u''], 
                 url=u'', open=u'', description=u'', 
                 tags=[u''], rss=u'', frequency=0, visit=0, up=0):
        
        self.url = url
        self.open = open
        self.description = description
        self.tags = tags
        self.rss = rss
        self.frequency = frequency
        self.up = up
        
        self.set(_text=text, _kanas=kanas, _visit=visit, _frequency=frequency)
        
    def up(self, data=None):
        if data>-1:
            self.up = data
        
    def __call__(self):
        Base.__call__(self)
        if GetOption(OPTION_BM_OPEN) and self.open:
            return self.open
        else:
            return self.url
    
    def showInfo(self):
        res = []
        res.append(self.baseInfo())
        res.append('url : ' + self.url)
        res.append('open : ' + self.open)
        res.append('description : ' + self.description)
        res.append('tags : ' + '|'.join(self.tags))
        res.append('up : ' + str(self.up))
        res.append('rss : ' + self.rss)
        return '\n'.join(res)
    
    def getStr(self, arg=None):
        text = []
        
        if self.up > self.visit:
            text.append(u'■')
        else:
            text.append(u'□')
            
        if self.up == 0:
            text.append(u'00/00 00:00 ')
        else:
            time = str(self.up)
            text.append(unicode(time[4:6] +'/'+ time[6:8] +' '+ time[8:10] +':'+ time[10:12] +' '))
            
        if len(self.description) > 0:
            text.append(self.text + u' ( ' +self.description + u' )')
        else:
            text.append(self.text)
            
        return ''.join(text)
    
    def checkDuplication(self):
        for old in self.__class__._items:
            if old.text.lower() == self.text.lower() and old.kanas == self.kanas and old.url.lower() == self.url.lower() and old.description.lower() == self.description.lower() and old.tags == self.tags:
                return 1
            elif old.url.lower() == self.url.lower():
                return old
            else:
                pass
    
    def makeOverwriteMsg(self, old):
        msg,msgtitle = Base.makeOverwriteMsg(self, old)
        
        _nopen = self.open
        _oopen = old.open
        msg.extend(['open url : ' + _oopen + '\n','open url : ' + _nopen + '\n\n'])
        if not _nopen.lower() == _oopen.lower():
            msg[-1], msg[-2] = '* '+msg[-1], '* '+msg[-2]
            
        _odes = old.description
        _ndes = self.description
        msg.extend(['description : ' + _odes + '\n','description : ' + _ndes + '\n\n'])
        if not _ndes.lower() == _odes.lower():
            msg[-1], msg[-2] = '* '+msg[-1], '* '+msg[-2]
        
        _otags = '|'.join(old.tags)
        _ntags = '|'.join(self.tags)
        msg.extend(['tag : ' + _otags + '\n','tag : ' + _ntags + '\n\n'])
        if not _ntags.lower() == _otags.lower():
            msg[-1], msg[-2] = '* '+msg[-1], '* '+msg[-2]
            
        _orss = old.rss
        _nrss = self.rss
        if _orss or _nrss:
            msg.extend(['rss : ' + _orss + '\n','rss : ' + _nrss + '\n\n'])
            if not _nrss.lower() == _orss.lower():
                msg[-1], msg[-2] = '* '+msg[-1], '* '+msg[-2]
            
        return msg,msgtitle
